import io
import os
import functools

import six
import yaml
from mako.template import Template
from zope.interface import implementer
from zope.component import IFactory

from .interfaces import (
    IParser,
    IPostProcess,
    IPreProcess,
    IReader,
    )


NAME = 'syaml'


@implementer(IPreProcess)
class SyamlPreProcess(object):
    def __call__(self, fileobj):
        if isinstance(fileobj, six.string_types):
            filepath = fileobj
            tmpl = Template(filename=filepath)
        else:  # may be file like object
            filepath = getattr(fileobj, 'name', None)  # fileobj has not name if it not file object
            tmpl = Template(fileobj.read())

        kwds = dict(os.environ)
        if filepath is not None:  # file like object, but not file object
            abs_file_path = os.path.abspath(filepath)
            kwds['here'] = os.path.dirname(abs_file_path)
            kwds['name'] = os.path.basename(abs_file_path)
            kwds['path'] = abs_file_path
        else:
            kwds['here'] = '""'
            kwds['name'] = '""'
            kwds['path'] = '""'
        buf = tmpl.render(**kwds)
        fp = io.BytesIO(buf.encode())
        fp.seek(0)
        return fp


@implementer(IParser)
class SyamlParser(object):
    def __call__(self, fileobj):
        return yaml.load(fileobj.read())


@implementer(IPostProcess)
class SyamlPostProcess(object):
    def __call__(self, obj):
        return obj


@implementer(IReader)
class SyamlReader(object):
    def __init__(self, pre, parse, post):
        self.pre = pre
        self.parse = parse
        self.post = post

    def __call__(self, fileobj):
        return functools.reduce(
            lambda obj, fun: fun(obj),
            (self.pre, self.parse, self.post),
            fileobj,
            )


@implementer(IFactory)
class SyamlReaderFactory(object):
    def __call__(self):
        return SyamlReader(
            pre=SyamlPreProcess(),
            parse=SyamlParser(),
            post=SyamlPostProcess(),
            )

get_reader = SyamlReaderFactory()
load = get_reader()


def includeme(config):
    reg = config.registry
    create_reader = SyamlReaderFactory()
    reader = create_reader()
    reg.registerUtility(reader, IReader, NAME)



import os
import logging

import yaml
from mako.template import Template
from zope.interface import implementer
from zope.component import IFactory

from .volumes import Volume
from .interfaces import ISetting

_logger = logging.getLogger(__name__)


@implementer(ISetting)
class VolumeSetting(dict):
    def __init__(self, *args, **kwds):
        super(VolumeSetting, self).__init__(*args, **kwds)
        self.project_name = None  # docker-compose project
        self.name = None
        self.dry_run = False

    def get_volume_name(self, name):
        if name not in self:
            raise KeyError(name)
        return '{}_{}'.format(self.project_name, name)

    def set_name(self, name):
        self.name = name

    def set_dry_run(self):
        self.dry_run = True


@implementer(IFactory)
class VolumeSettingParser(object):
    def __init__(self, env, path, project_name=''):
        self.env = env
        self.path = path
        self.project_name = project_name

    def __call__(self, env=None, path=None, project_name=None):
        env = self.env if env is None else env
        path = self.path if path is None else path
        project_name = self.project_name if project_name is None else project_name

        kwds = dict(os.environ)
        kwds['here'] = os.path.abspath(os.path.dirname(path))

        tmpl = Template(filename=path)
        data = yaml.load(tmpl.render(**kwds))

        setting = VolumeSetting()
        setting.set_name(env)
        setting.project_name = project_name
        for volume_name, volume_data in data['volumes'].items():
            volume = Volume(**volume_data)
            setting[volume_name] = volume
        return setting



from zope.interface import (
    Attribute,
    Interface,
    )


class IPreProcess(Interface):
    def __call__(fileobj):
        """Initial processing for target file.

        :param fileobj:
        :rtype: file like object
        :return fileobj: file object (seeked to top)
        """


class IParser(Interface):
    def __call__(fileobj):
        """Parse fileobj

        :param fileobj:
        :rtype: unkown
        :return obj: parsed object
        """


class IPostProcess(Interface):
    def __call__(obj):
        """Post process to obj

        :param fileobj:
        :rtype: unkown
        :return obj: post processed object
        """


class IReader(Interface):
    pre = Attribute('pre process')
    parse = Attribute('parser process')
    post = Attribute('post process')

    def __call__(obj):
        """Read formatted file"""
